﻿namespace DbContextWrapperExample.Entities
{
    using System;
    using System.Collections.Generic;
    using System.Data.SqlClient;
    using System.Threading.Tasks;

    public partial class Jiggy
    {
        private object ValueOrDbNull(object input)
        {
            return input ?? DBNull.Value;
        }
        #region StoredProcedures

        public async Task<List<StuffSelect_Result>> StuffSelectAsync(Guid stuffGuid)
        {
            return await this.Database.SqlQuery<StuffSelect_Result>(
                sql: "[dbo].[StuffSelect] @StuffGuid ",
                parameters: new object[]
                {
                    new SqlParameter(parameterName: "@StuffGuid", value: ValueOrDbNull(stuffGuid))
                })
                .ToListAsync()
                .ConfigureAwait(false);
        }

        public async Task<List<StuffFind_Result>> StuffFindAsync(Guid? stuffGuid = null, string name = null, Int32? size = null)
        {
            return await this.Database.SqlQuery<StuffFind_Result>(
                sql: "[dbo].[StuffFind] @StuffGuid, @Name, @Size ",
                parameters: new object[]
                {
                    new SqlParameter(parameterName: "@StuffGuid", value: ValueOrDbNull(stuffGuid)),
                    new SqlParameter(parameterName: "@Name", value: ValueOrDbNull(name)),
                    new SqlParameter(parameterName: "@Size", value: ValueOrDbNull(size))
                })
                .ToListAsync()
                .ConfigureAwait(false);
        }

        public async Task<List<StuffSearch_Result>> StuffSearchAsync(string nameStartsWith = null, string nameContains = null, Int32? sizeGreaterThan = null, Int32? sizeLessThan = null)
        {
            return await this.Database.SqlQuery<StuffSearch_Result>(
                sql: "[dbo].[StuffSearch] @NameStartsWith, @NameContains, @SizeGreaterThan, @SizeLessThan ",
                parameters: new object[]
                {
                    new SqlParameter(parameterName: "@NameStartsWith", value: ValueOrDbNull(nameStartsWith)),
                    new SqlParameter(parameterName: "@NameContains", value: ValueOrDbNull(nameContains)),
                    new SqlParameter(parameterName: "@SizeGreaterThan", value: ValueOrDbNull(sizeGreaterThan)),
                    new SqlParameter(parameterName: "@SizeLessThan", value: ValueOrDbNull(sizeLessThan))
                })
                .ToListAsync()
                .ConfigureAwait(false);
        }

        public async Task<int> StuffUpdateAsync(Guid stuffGuid, string name, Int32 size)
        {
            return await this.Database.ExecuteSqlCommandAsync(
                sql: "[dbo].[StuffUpdate] @StuffGuid, @Name, @Size ",
                parameters: new object[]
                {
                    new SqlParameter(parameterName: "@StuffGuid", value: ValueOrDbNull(stuffGuid)),
                    new SqlParameter(parameterName: "@Name", value: ValueOrDbNull(name)),
                    new SqlParameter(parameterName: "@Size", value: ValueOrDbNull(size))
                })
                .ConfigureAwait(false);
        }

        public async Task<int> StuffInsertAsync(Guid stuffGuid, string name, Int32 size)

        {
            return await this.Database.ExecuteSqlCommandAsync(
                sql: "[dbo].[StuffInsert] @StuffGuid, @Name, @Size ",
                parameters: new object[]
                {
                    new SqlParameter(parameterName: "@StuffGuid", value: ValueOrDbNull(stuffGuid)),
                    new SqlParameter(parameterName: "@Name", value: ValueOrDbNull(name)),
                    new SqlParameter(parameterName: "@Size", value: ValueOrDbNull(size))
                })
                .ConfigureAwait(false);
        }

        public async Task<int> StuffDeleteAsync(Guid stuffGuid)
        {
            return await this.Database.ExecuteSqlCommandAsync(
                sql: "[dbo].[StuffDelete] @StuffGuid ",
                parameters: new object[]
                {
                    new SqlParameter(parameterName: "@StuffGuid", value: ValueOrDbNull(stuffGuid))
                })
                .ConfigureAwait(false);
        }

        public async Task<List<ThingSelect_Result>> ThingSelectAsync(Guid thingGuid)
        {
            return await this.Database.SqlQuery<ThingSelect_Result>(
                sql: "[dbo].[ThingSelect] @ThingGuid ",
                parameters: new object[]
                {
                    new SqlParameter(parameterName: "@ThingGuid", value: ValueOrDbNull(thingGuid))
                })
                .ToListAsync()
                .ConfigureAwait(false);
        }

        public async Task<List<ThingFind_Result>> ThingFindAsync(Guid? thingGuid = null, Guid? stuffGuid = null, string name = null, string description = null)
        {
            return await this.Database.SqlQuery<ThingFind_Result>(
                sql: "[dbo].[ThingFind] @ThingGuid, @StuffGuid, @Name, @Description ",
                parameters: new object[]
                {
                    new SqlParameter(parameterName: "@ThingGuid", value: ValueOrDbNull(thingGuid)),
                    new SqlParameter(parameterName: "@StuffGuid", value: ValueOrDbNull(stuffGuid)),
                    new SqlParameter(parameterName: "@Name", value: ValueOrDbNull(name)),
                    new SqlParameter(parameterName: "@Description", value: ValueOrDbNull(description))
                })
                .ToListAsync()
                .ConfigureAwait(false);
        }

        public async Task<List<ThingSearch_Result>> ThingSearchAsync(string nameStartsWith = null, string nameContains = null, string descriptionStartsWith = null, string descriptionContains = null)
        {
            return await this.Database.SqlQuery<ThingSearch_Result>(
                sql: "[dbo].[ThingSearch] @NameStartsWith, @NameContains, @DescriptionStartsWith, @DescriptionContains ",
                parameters: new object[]
                {
                    new SqlParameter(parameterName: "@NameStartsWith", value: ValueOrDbNull(nameStartsWith)),
                    new SqlParameter(parameterName: "@NameContains", value: ValueOrDbNull(nameContains)),
                    new SqlParameter(parameterName: "@DescriptionStartsWith",
                        value: ValueOrDbNull(descriptionStartsWith)),
                    new SqlParameter(parameterName: "@DescriptionContains", value: ValueOrDbNull(descriptionContains))
                })
                .ToListAsync()
                .ConfigureAwait(false);
        }

        public async Task<int> ThingUpdateAsync(Guid thingGuid, Guid stuffGuid, string name, string description)
        {
            return await this.Database.ExecuteSqlCommandAsync(
                sql: "[dbo].[ThingUpdate] @ThingGuid, @StuffGuid, @Name, @Description ",
                parameters: new object[]
                {
                    new SqlParameter(parameterName: "@ThingGuid", value: ValueOrDbNull(thingGuid)),
                    new SqlParameter(parameterName: "@StuffGuid", value: ValueOrDbNull(stuffGuid)),
                    new SqlParameter(parameterName: "@Name", value: ValueOrDbNull(name)),
                    new SqlParameter(parameterName: "@Description", value: ValueOrDbNull(description))
                })
                .ConfigureAwait(false);
        }

        public async Task<int> ThingInsertAsync(Guid thingGuid, Guid stuffGuid, string name, string description)
        {
            return await this.Database.ExecuteSqlCommandAsync(
                sql: "[dbo].[ThingInsert] @ThingGuid, @StuffGuid, @Name, @Description ",
                parameters: new object[]
                {
                    new SqlParameter(parameterName: "@ThingGuid", value: ValueOrDbNull(thingGuid)),
                    new SqlParameter(parameterName: "@StuffGuid", value: ValueOrDbNull(stuffGuid)),
                    new SqlParameter(parameterName: "@Name", value: ValueOrDbNull(name)),
                    new SqlParameter(parameterName: "@Description", value: ValueOrDbNull(description))
                })
                .ConfigureAwait(false);
        }

        public async Task<int> ThingDeleteAsync(Guid thingGuid)
        {
            return await this.Database.ExecuteSqlCommandAsync(
                sql: "[dbo].[ThingDelete] @ThingGuid ",
                parameters: new object[]
                {
                    new SqlParameter(parameterName: "@ThingGuid", value: ValueOrDbNull(thingGuid))
                })
                .ConfigureAwait(false);
        }
        #endregion
    }
}
